# WGS坐标系经纬度与ECEF直角坐标的基本换算



# 1. WGS-84 坐标系



WGS 坐标系是目前使用最为广泛的全球坐标系统，全称为世界大地测量系统（英语：World Geodetic System）是一种用于地图学、大地测量学和导航（包括全球定位系统）的大地测量系统标准。WGS包含一套地球的标准经纬坐标系、一个用于计算原始海拔数据的参考椭球体，和一套用以定义海平面高度的引力等势面数据。其示意图如下：



![WGS84坐标系](../images/ecef_wgs84.svg)



WGS坐标系里，地球等效为一个椭圆，直角坐标系原点O位于地球的中心。



- 坐标轴$$$\vec{Z} =\vec{ON}$$$指向北极点。

- 坐标轴$$$\vec{X} =\vec{OP}$$$指向格林威治0度经线与赤道面的交点P。

- 坐标轴$$$\vec Y=\vec X\otimes \vec Z$$$呈右手螺旋关系。



当前位置 T 的经纬度定义：



从高度 H 开始定义：



- T是一个距离参考海平面（椭球面）高度为$$$\vec{H} =\vec{tT}$$$的点。

- t是点T在椭球面上的正射投影，矢量$$$\vec{tT}$$$是椭球面上t点切面的法向量。

- 需要注意的是，和正球面不同，法向量$$$\vec{tT}$$$的反向延长线与Z轴的交点是 O'，和原点O并不重合。



对于经度$$$\theta$$$的定义，是这样的：



- 0度经度大圆$$$\stackrel\frown{NOP}$$$是由北极点 N, 地心 O, 格林威治经线与赤道面的交点 P 组成的平面。

- 当前经度大圆$$$\stackrel\frown{NO'T}$$$是由北极点 N, 交点O', 当前位置T组成的平面。

- O、 O' 、t、 T 都在当前经度面上，直线 O'T 与赤道面的交点为u，矢量$$$\vec{Ou}$$$与赤道的交点是Q。

- O'、u、t、T四点共线。

- 经度$$$\theta=\measuredangle {POQ}$$$





对于纬度$$$\varphi$$$的定义，是这样的：



- 椭球的切平面的法线与Z的交点 O' 和原点并不重合。

- 纬度定义为法向量$$$\vec{uT}$$$与赤道面的夹角。在北半球为正，南半球为负

- 纬度$$$\varphi=\measuredangle {TuQ}=\measuredangle {OuO'}$$$



## 1.1 从经纬度转换到ECEF



- ECEF 坐标系有利于利用矢量计算简化角度、距离的求取。



已知:



- 赤道半径 a = 6378137 米，对应上图$$$\vec{OP}$$$的长度;

- 椭球扁率$$$f = (a-b)/a$$$= 1.0/298.257223563。扁率就是衡量椭球是不是很“扁”，正球的扁率为0.

- 椭球偏心率$$$e={\sqrt{a^2-b^2} }/{a}$$$= 0.0818191908426

- e和f的关系为$$$e^2=f(2-f)$$$

- 极地半径 b = 6356752.3142 米，对应上图$$$\vec{ON}$$$的长度; 



设：经度$$$\theta$$$， 纬度$$$\varphi$$$， 海拔 H，则有：



$$r = \left | \vec R\right |=\left | \vec{O't}  \right | = \frac{ a}{\sqrt{1-e^2 \sin^2\varphi }} =\frac{ a}{\sqrt{1-f(2-f) \sin^2\varphi }}$$



经纬度极坐标可以利用矢量$$$\vec R$$$构造一个Z轴平移了的正球，进行极坐标运算。这个正球的球心是 O'。由于x,y没有平移，故而两个坐标系的x、y是重合的。Z坐标z'与z的关系是平移，平移量为OO'的长度：



$$d=\vec {OO'} = - re^2\sin \varphi$$



下图是把经度大圆$$$\stackrel\frown {NOQ}$$$切割出来，观察：



![WGS84坐标系](../images/ecef_sphere.svg)



在正球模型下，根据极坐标的基本定义，可以直接计算x,y:



$$x=\left ( r + H \right ) \cdot \cos \varphi \cdot cos \theta$$



$$y=\left ( r + H \right ) \cdot \cos \varphi \cdot sin \theta$$



Z的坐标，要先计算后，再平移即可得到:



$$z'=\left ( r + H \right ) \cdot sin \varphi$$

$$z=z' + d = z' - re^2\sin \varphi$$



带入后，可写成如下等效形式：



$$z=(r(1-e^2)+H) \sin \varphi$$

$$z=(r(1-f(2-f))+H) \sin \varphi =(r(1-f)^2+H) \sin \varphi$$



含有扁率的化简，是基于扁率$$$f = (a-b)/a = 1 - b/a$$$计算得到的。



通过上面的处理，极坐标的$$$T(\theta, \varphi,H)$$$便转换为平面直角坐标$$$T(x,y,z)$$$。



软件接口：



```cpp

/*!

 * \brief lla2ecef 经纬度坐标到ECEF，

 * \param lla  纬经高(默认)/经纬高, 量纲是度(默认)/弧度、米

 * \param ecef xyz，量纲是米

 * \param pr 传出正球半径r,量纲是米

 * \param pd 传出正球原点偏移 OO' d 量纲是米

 * \param rad  角度量纲开关，false 是度，true 是弧度

 * \param latfirst 经纬度顺序，false 是经度\纬度\高度，true 是纬度\经度\高度

*/

inline void lla2ecef(

	const double lla[/*3*/],

	double ecef[/*3*/],

	double * pr = nullptr,

	double * pd = nullptr,

	const bool rad = false,

	const bool latfirst = true

	);

```





## 1.2 从 ECEF 到经纬度



对上述计算过程而言，逆向运算可以立刻得到经度，却不能解析得到纬度。纬度需要进行迭代。



主要原因是：



$$d=\vec {OO'} = -re^2\sin \varphi$$



的具体取值不知道导致的。



### (1) 求取经度



由于



$$x/y=\tan \theta$$



故而



$$\theta=\tan^{-1} (x/y)$$



注意处理好象限问题，即可得到经度。



### (2) 纬度、海拔计算



对于纬度，则比较复杂。求解三角方程



$$(r + H) \sin \varphi = z - d$$



由于 H 不知道，r 也不知道，加入第二条件（相切，H向量与r共线）后，得到的是二元四次三角方程，很难解析求取。



可以首先假设$$$\left | \vec {OO'} \right | =0$$$, 求取一个粗略的纬度，并得到 d，利用d反过来更新其他数值，进行迭代。这种迭代收敛的前提，是因为每次计算出的纬度一定小于真实的纬度，这是由三角关系决定的。



![WGS84坐标系](../images/ecef_calc.svg)



迭代的效果是不断地把原点向真实的原点移动，直到z达到一个很小的误差。迭代方法：



设起始偏移$$$d = 0$$$，迭代开始：



$$R'=r+H = \sqrt{x^2+y^2+(z-d)^2}$$

$$\varphi' = \sin^{-1} {\frac {z-d}{R'}}$$

$$r' = \frac{ a}{\sqrt{1-e^2 \sin^2\varphi' }}$$

$$d'= -r'e^2\sin \varphi'$$

$$z'=R'sin \varphi'+d'$$

$$d = d'$$



当 

$$err = \left | z-z' \right | < \varepsilon$$

时停止迭代。此时



$$\varphi = \sin^{-1} {\frac {z-d}{\sqrt{x^2+y^2+(z-d)^2}}}$$



$$H= R'-r'$$



软件接口：

```cpp

/*!

 * \brief ecef2lla ECEF到经纬度坐标

 * \param ecef xyz，量纲是米

 * \param lla  纬经高(默认)/经纬高, 量纲是度(默认)/弧度、米

 * \param pr 传出正球半径r,量纲是米

 * \param pd 传出正球原点偏移 OO' d 量纲是米

 * \param maxiter  最大迭代次数

 * \param piter  迭代次数输出，可以为null

 * \param eps  Z误差门限

 * \param rad  角度量纲开关，false 是度，true 是弧度

 * \param latfirst 经纬度顺序，false 是经度\纬度\高度，true 是纬度\经度\高度

 * \return 迭代收敛标志

*/

inline bool ecef2lla(

	const double ecef[/*3*/],

	double lla[/*3*/],

	double * pr = nullptr,

	double * pd = nullptr,

	const int maxiter = 32,

	int * piter = nullptr,

	const double eps = 1e-10,

	const bool rad = false,

	const bool latfirst = true

	)

```


